@@ -1,16 +1,48 @@
+#![allow(missing_docs)]
+
 fn main() {
     #[cfg(feature = "german")]
     natural_languages::generate_word_lists();
+
+    hcl::build();
+}
+
+mod hcl {
+    /// The function body is mostly the output of `tree-sitter generate` (`tree-sitter`
+    /// version 0.22.5) inside of
+    /// <https://github.com/tree-sitter-grammars/tree-sitter-hcl>, see also
+    /// <https://tree-sitter.github.io/tree-sitter/creating-parsers#command-generate>.
+    /// The resulting `bindings/rust/build.rs` is the below code, slimmed down to only
+    /// what's strictly needed, e.g. not including any warning flags.
+    ///
+    /// **Remove this code once
+    /// <https://github.com/tree-sitter-grammars/tree-sitter-hcl> is available on
+    /// <https://crates.io> and updated to a high enough `tree-sitter` version**.
+    pub fn build() {
+        let src_dir = std::path::Path::new("src/scoping/langs/tree_sitter_hcl/upstream-main/src");
+
+        let mut c_config = cc::Build::new();
+        c_config.include(src_dir);
+        let parser_path = src_dir.join("parser.c");
+        c_config.file(&parser_path);
+        println!("cargo:rerun-if-changed={}", parser_path.to_str().unwrap());
+
+        let scanner_path = src_dir.join("scanner.c");
+        c_config.file(&scanner_path);
+        println!("cargo:rerun-if-changed={}", scanner_path.to_str().unwrap());
+
+        c_config.warnings(false);
+        c_config.compile("parser");
+    }
 }
 
 #[cfg(feature = "german")]
+#[allow(unreachable_pub)] // Cannot get this to play nice with clippy
 mod natural_languages {
+    use std::env;
+    use std::fs::{self, File};
     use std::io::{BufReader, BufWriter};
-    use std::{
-        env,
-        fs::{self, File},
-        path::Path,
-    };
+    use std::path::Path;
 
     pub fn generate_word_lists() {
         let base_source_path = Path::new("data/word-lists");
@@ -24,27 +56,36 @@ mod natural_languages {
         {
             // German
             let source_file = base_source_path.join("de.txt");
-            let destination_file = base_destination_path.join("de.fst");
-            destination_file.parent().map(fs::create_dir_all);
+            println!("cargo::rerun-if-changed={}", source_file.display());
 
-            german::process_german(
+            let destination_file = base_destination_path.join("de.fst");
+            destination_file
+                .parent()
+                .map(|p| fs::create_dir_all(p).expect("directory creation to succeed"))
+                .expect("parent directory to be present");
+
+            if destination_file.exists() {
+                println!("Output file already exists, skipping generation");
+                return;
+            }
+
+            german::process(
                 &mut BufReader::new(File::open(&source_file).unwrap()),
                 &mut BufWriter::new(File::create(destination_file).unwrap()),
             );
-
-            println!("cargo:rerun-if-changed={}", source_file.display());
         }
     }
 
     #[cfg(feature = "german")]
     mod german {
-        use decompound::{decompound, DecompositionOptions};
-        use rayon::prelude::*;
         use std::collections::HashSet;
         use std::env;
         use std::io::{BufReader, BufWriter, Read, Write};
         use std::sync::Mutex;
 
+        use decompound::{decompound, DecompositionOptions};
+        use rayon::prelude::*;
+
         macro_rules! time_it {
             ($name:expr, $e:expr) => {{
                 let now = std::time::Instant::now();
@@ -55,17 +96,17 @@ mod natural_languages {
             }};
         }
 
-        pub fn process_german<R, W>(source: &mut BufReader<R>, destination: &mut BufWriter<W>)
+        pub fn process<R, W>(source: &mut BufReader<R>, destination: &mut BufWriter<W>)
         where
             R: Read,
             W: Write,
         {
             let mut contents = String::new();
-            source.read_to_string(&mut contents).unwrap();
+            let _ = source.read_to_string(&mut contents).unwrap();
 
             let words: HashSet<&str> = time_it!(
                 "Constructing hashset of words",
-                contents.lines().map(|word| word.trim()).collect()
+                contents.lines().map(str::trim).collect()
             );
             let keepers = Mutex::new(Vec::with_capacity(words.len()));
 
@@ -74,6 +115,7 @@ mod natural_languages {
                 // Parallel iteration is a massive time-saver, more than an order of magnitude
                 // (approx. 2 minutes -> 5 seconds)
                 words.par_iter().for_each(|word| {
+                    #[allow(clippy::single_match_else)]
                     match decompound(
                         word,
                         &|w| words.contains(w),
@@ -96,8 +138,8 @@ mod natural_languages {
 
             let mut keepers = keepers.into_inner().unwrap();
             let dropped_words: HashSet<_> = words
-                .difference(&keepers.iter().cloned().collect::<HashSet<_>>())
-                .cloned()
+                .difference(&keepers.iter().copied().collect::<HashSet<_>>())
+                .copied()
                 .collect();
 
             drop(words); // Prevent misuse; these are unfiltered!
@@ -105,19 +147,19 @@ mod natural_languages {
             let n_dropped = dropped_words.len();
             if n_dropped > 0 {
                 println!(
-                    "cargo:warning=Dropped {} compound words ({} remaining); see '{:?}' for a list.",
+                    "cargo::warning=Dropped {} compound words ({} remaining); see '{:?}' for a list.",
                     n_dropped,
                     keepers.len(),
                     {
                         let mut path: std::path::PathBuf = env::var_os("OUT_DIR").unwrap().into();
-                        path.pop(); // Remove "out"
+                        assert!(path.pop(), "no parent element"); // Remove "out"
                         path.push("output"); // The log file
                         path
                     },
                 );
             }
 
-            time_it!("Sorting filtered words", keepers.sort());
+            time_it!("Sorting filtered words", keepers.sort_unstable());
 
             // `fst::SetBuilder.insert` doesn't check for dupes, so be sure (?)
             time_it!("Deduplicating filtered words", keepers.dedup());
