@@ -23,6 +23,7 @@ use srgn::actions::{
 };
 #[cfg(feature = "symbols")]
 use srgn::actions::{Symbols, SymbolsInversion};
+use srgn::scoping::langs::c::{CQuery, C};
 use srgn::scoping::langs::csharp::{CSharp, CSharpQuery};
 use srgn::scoping::langs::go::{Go, GoQuery};
 use srgn::scoping::langs::hcl::{Hcl, HclQuery};
@@ -334,6 +335,7 @@ fn handle_actions_on_many_files_sorted(
 
 /// Main entrypoint for processing using at least 1 thread.
 #[allow(clippy::borrowed_box)] // Used throughout, not much of a pain
+#[allow(clippy::too_many_lines)]
 fn handle_actions_on_many_files_threaded(
     validator: &Validator,
     general_scoper: &Box<dyn Scoper>,
@@ -440,7 +442,8 @@ fn handle_actions_on_many_files_threaded(
             })
         });
 
-    if let Some(e) = err.lock().unwrap().take() {
+    let error = err.lock().unwrap().take();
+    if let Some(e) = error {
         return Err(e);
     }
 
@@ -828,6 +831,7 @@ fn get_language_scopers(args: &cli::Cli) -> Vec<Box<dyn LanguageScoper>> {
         };
     }
 
+    handle_language_scope!(c, c_query, CQuery, C);
     handle_language_scope!(csharp, csharp_query, CSharpQuery, CSharp);
     handle_language_scope!(hcl, hcl_query, HclQuery, Hcl);
     handle_language_scope!(go, go_query, GoQuery, Go);
@@ -934,6 +938,7 @@ mod cli {
     use clap::builder::ArgPredicate;
     use clap::{ArgAction, Command, CommandFactory, Parser};
     use clap_complete::{generate, Generator, Shell};
+    use srgn::scoping::langs::c::{CustomCQuery, PreparedCQuery};
     use srgn::scoping::langs::csharp::{CustomCSharpQuery, PreparedCSharpQuery};
     use srgn::scoping::langs::go::{CustomGoQuery, PreparedGoQuery};
     use srgn::scoping::langs::hcl::{CustomHclQuery, PreparedHclQuery};
@@ -1222,6 +1227,8 @@ mod cli {
     #[group(required = false, multiple = false)]
     #[command(next_help_heading = "Language scopes")]
     pub struct LanguageScopes {
+        #[command(flatten)]
+        pub c: Option<CScope>,
         #[command(flatten)]
         pub csharp: Option<CSharpScope>,
         #[command(flatten)]
@@ -1236,11 +1243,23 @@ mod cli {
         pub typescript: Option<TypeScriptScope>,
     }
 
+    #[derive(Parser, Debug, Clone)]
+    #[group(required = false, multiple = false)]
+    pub struct CScope {
+        /// Scope C code using a prepared query.
+        #[arg(long, env, verbatim_doc_comment)]
+        pub c: Vec<PreparedCQuery>,
+
+        /// Scope C code using a custom tree-sitter query.
+        #[arg(long, env, verbatim_doc_comment, value_name = TREE_SITTER_QUERY_VALUE_NAME)]
+        pub c_query: Vec<CustomCQuery>,
+    }
+
     #[derive(Parser, Debug, Clone)]
     #[group(required = false, multiple = false)]
     pub struct CSharpScope {
         /// Scope C# code using a prepared query.
-        #[arg(long, env, verbatim_doc_comment)]
+        #[arg(long, env, verbatim_doc_comment, visible_alias = "cs")]
         pub csharp: Vec<PreparedCSharpQuery>,
 
         /// Scope C# code using a custom tree-sitter query.
@@ -1278,7 +1297,7 @@ mod cli {
     #[group(required = false, multiple = false)]
     pub struct PythonScope {
         /// Scope Python code using a prepared query.
-        #[arg(long, env, verbatim_doc_comment)]
+        #[arg(long, env, verbatim_doc_comment, visible_alias = "py")]
         pub python: Vec<PreparedPythonQuery>,
 
         /// Scope Python code using a custom tree-sitter query.
@@ -1290,7 +1309,7 @@ mod cli {
     #[group(required = false, multiple = false)]
     pub struct RustScope {
         /// Scope Rust code using a prepared query.
-        #[arg(long, env, verbatim_doc_comment)]
+        #[arg(long, env, verbatim_doc_comment, visible_alias = "rs")]
         pub rust: Vec<PreparedRustQuery>,
 
         /// Scope Rust code using a custom tree-sitter query.
@@ -1302,7 +1321,7 @@ mod cli {
     #[group(required = false, multiple = false)]
     pub struct TypeScriptScope {
         /// Scope TypeScript code using a prepared query.
-        #[arg(long, env, verbatim_doc_comment)]
+        #[arg(long, env, verbatim_doc_comment, visible_alias = "ts")]
         pub typescript: Vec<PreparedTypeScriptQuery>,
 
         /// Scope TypeScript code using a custom tree-sitter query.
