@@ -1,19 +1,22 @@
-use crate::actions::{self, Action, ReplacementCreationError};
-use crate::scoping::dosfix::DosFix;
-use crate::scoping::scope::{
-    ROScope, ROScopes, RWScope, RWScopes,
-    Scope::{In, Out},
-};
-use crate::scoping::Scoper;
-use log::{debug, trace, warn};
 use std::borrow::Cow;
 use std::fmt;
 
+use itertools::Itertools;
+use log::{debug, trace, warn};
+
+use crate::actions::{self, Action, ActionError};
+use crate::scoping::dosfix::DosFix;
+use crate::scoping::scope::Scope::{In, Out};
+#[cfg(doc)]
+use crate::scoping::scope::ScopeContext;
+use crate::scoping::scope::{ROScope, ROScopes, RWScope, RWScopes};
+use crate::scoping::Scoper;
+
 /// A view of some input, sorted into parts, which are either [`In`] or [`Out`] of scope
 /// for processing.
 ///
-/// The view is **writable**. It can be manipulated by [mapping][`Self::map`]
-/// [`Action`]s over it.
+/// The view is **writable**. It can be manipulated by
+/// [mapping][`Self::map_without_context`] [`Action`]s over it.
 ///
 /// The main avenue for constructing a view is [`Self::builder`].
 #[derive(Debug, Clone, PartialEq, Eq)]
@@ -25,10 +28,16 @@ pub struct ScopedView<'viewee> {
 impl<'viewee> ScopedView<'viewee> {
     /// Create a new view from the given scopes.
     #[must_use]
-    pub fn new(scopes: RWScopes<'viewee>) -> Self {
+    pub const fn new(scopes: RWScopes<'viewee>) -> Self {
         Self { scopes }
     }
 
+    /// Access the scopes contained in this view.
+    #[must_use]
+    pub const fn scopes(&self) -> &RWScopes<'viewee> {
+        &self.scopes
+    }
+
     /// Return a builder for a view of the given input.
     ///
     /// For API discoverability.
@@ -40,19 +49,48 @@ impl<'viewee> ScopedView<'viewee> {
     /// Apply an `action` to all [`In`] scope items contained in this view.
     ///
     /// They are **replaced** with whatever the action returns for the particular scope.
+    /// This method is infallible, as it does not access any [`ScopeContext`].
     ///
     /// See implementors of [`Action`] for available types.
-    pub fn map(&mut self, action: &impl Action) -> &mut Self {
+    #[allow(clippy::missing_panics_doc)] // 🤞
+    pub fn map_without_context(&mut self, action: &impl Action) -> &mut Self {
+        self.map_impl(action, false)
+            .expect("not accessing context, so is infallible");
+
+        self
+    }
+
+    /// Same as [`Self::map_without_context`], but will access any [`ScopeContext`],
+    /// which is fallible.
+    ///
+    /// # Errors
+    ///
+    /// See the concrete type of the [`Err`] variant for when this method errors.
+    pub fn map_with_context(&mut self, action: &impl Action) -> Result<&mut Self, ActionError> {
+        self.map_impl(action, true)?;
+
+        Ok(self)
+    }
+
+    fn map_impl(
+        &mut self,
+        action: &impl Action,
+        use_context: bool,
+    ) -> Result<&mut Self, ActionError> {
         for scope in &mut self.scopes.0 {
             match scope {
-                RWScope(In(s)) => {
-                    let res = action.act(s);
+                RWScope(In(s, ctx)) => {
+                    debug!("Mapping with context: {:?}", ctx);
+                    let res = match (&ctx, use_context) {
+                        (Some(c), true) => action.act_with_context(s, c)?,
+                        _ => action.act(s),
+                    };
                     debug!(
                         "Replacing '{}' with '{}'",
                         s.escape_debug(),
                         res.escape_debug()
                     );
-                    *scope = RWScope(In(Cow::Owned(res)));
+                    *scope = RWScope(In(Cow::Owned(res), ctx.clone()));
                 }
                 RWScope(Out(s)) => {
                     debug!("Appending '{}'", s.escape_debug());
@@ -60,7 +98,7 @@ impl<'viewee> ScopedView<'viewee> {
             }
         }
 
-        self
+        Ok(self)
     }
 
     /// Squeeze all consecutive [`In`] scopes into a single occurrence (the first one).
@@ -69,8 +107,8 @@ impl<'viewee> ScopedView<'viewee> {
 
         let mut prev_was_in = false;
         self.scopes.0.retain(|scope| {
-            let keep = !(prev_was_in && matches!(scope, RWScope(In(_))));
-            prev_was_in = matches!(scope, RWScope(In(_)));
+            let keep = !(prev_was_in && matches!(scope, RWScope(In { .. })));
+            prev_was_in = matches!(scope, RWScope(In { .. }));
             trace!("keep: {}, scope: {:?}", keep, scope);
             keep
         });
@@ -84,86 +122,147 @@ impl<'viewee> ScopedView<'viewee> {
     #[must_use]
     pub fn has_any_in_scope(&self) -> bool {
         self.scopes.0.iter().any(|s| match s {
-            RWScope(In(_)) => true,
-            RWScope(Out(_)) => false,
+            RWScope(In { .. }) => true,
+            RWScope(Out { .. }) => false,
         })
     }
+
+    /// Split this item at newlines, into multiple [`ScopedView`]s.
+    ///
+    /// Scopes are retained, and broken across lines as needed.
+    #[must_use]
+    pub fn lines(&self) -> ScopedViewLines<'_> {
+        let mut lines = Vec::new();
+        let mut curr = Vec::new();
+
+        for parent_scope in &self.scopes.0 {
+            let s: &str = parent_scope.into();
+
+            for potential_line in s.split_inclusive('\n') {
+                // Is it supposed to be in or out of scope?
+                let child_scope = match &parent_scope.0 {
+                    In(_, ctx) => In(Cow::Borrowed(potential_line), ctx.clone()),
+                    Out(_) => Out(potential_line),
+                };
+
+                // String might not have *any* newlines, so this isn't redundant
+                let seen_newline = potential_line.ends_with('\n');
+
+                curr.push(RWScope(child_scope));
+
+                if seen_newline {
+                    // Flush out
+                    lines.push(RWScopes(curr));
+                    curr = Vec::new();
+                }
+            }
+        }
+
+        if !curr.is_empty() {
+            // Tail that wasn't flushed yet
+            lines.push(RWScopes(curr));
+        }
+
+        ScopedViewLines(lines.into_iter().map(ScopedView::new).collect_vec())
+    }
+}
+
+/// A view over a [`ScopedView`], split by its individual lines. Each line is its own
+/// [`ScopedView`].
+#[derive(Debug, Clone, PartialEq, Eq)]
+pub struct ScopedViewLines<'viewee>(Vec<ScopedView<'viewee>>);
+
+impl<'viewee> IntoIterator for ScopedViewLines<'viewee> {
+    type Item = ScopedView<'viewee>;
+    type IntoIter = std::vec::IntoIter<Self::Item>;
+
+    fn into_iter(self) -> Self::IntoIter {
+        self.0.into_iter()
+    }
 }
 
 /// Implementations of all available actions as dedicated methods.
 ///
 /// Where actions don't take arguments, neither do the methods.
-impl<'viewee> ScopedView<'viewee> {
-    /// Apply the default [`actions::Deletion`] action to this view (see [`Self::map`]).
+impl ScopedView<'_> {
+    /// Apply the default [`actions::Deletion`] action to this view (see
+    /// [`Self::map_without_context`]).
     pub fn delete(&mut self) -> &mut Self {
         let action = actions::Deletion::default();
 
-        self.map(&action)
+        self.map_without_context(&action)
     }
 
-    /// Apply the default [`actions::German`] action to this view (see [`Self::map`]).
+    /// Apply the default [`actions::German`] action to this view (see
+    /// [`Self::map_without_context`]).
     #[cfg(feature = "german")]
     pub fn german(&mut self) -> &mut Self {
         let action = actions::German::default();
 
-        self.map(&action)
+        self.map_without_context(&action)
     }
 
-    /// Apply the default [`actions::Lower`] action to this view (see [`Self::map`]).
+    /// Apply the default [`actions::Lower`] action to this view (see
+    /// [`Self::map_without_context`]).
     pub fn lower(&mut self) -> &mut Self {
         let action = actions::Lower::default();
 
-        self.map(&action)
+        self.map_without_context(&action)
     }
 
     /// Apply the default [`actions::Normalization`] action to this view (see
-    /// [`Self::map`]).
+    /// [`Self::map_without_context`]).
     pub fn normalize(&mut self) -> &mut Self {
         let action = actions::Normalization::default();
 
-        self.map(&action)
+        self.map_without_context(&action)
     }
 
-    /// Apply the [`actions::Replacement`] action to this view (see [`Self::map`]).
+    /// Apply the [`actions::Replacement`] action to this view (see
+    /// [`Self::map_with_context`]).
     ///
     /// ## Errors
     ///
     /// For why and how this can fail, see the implementation of [`TryFrom<String>`] for
     /// [`actions::Replacement`].
-    pub fn replace(&mut self, replacement: String) -> Result<&mut Self, ReplacementCreationError> {
+    pub fn replace(&mut self, replacement: String) -> Result<&mut Self, ActionError> {
         let action = actions::Replacement::try_from(replacement)?;
 
-        Ok(self.map(&action))
+        self.map_with_context(&action)
     }
 
-    /// Apply the [`actions::Symbols`] action to this view (see [`Self::map`]).
+    /// Apply the [`actions::Symbols`] action to this view (see
+    /// [`Self::map_without_context`]).
     #[cfg(feature = "symbols")]
     pub fn symbols(&mut self) -> &mut Self {
         let action = actions::Symbols::default();
 
-        self.map(&action)
+        self.map_without_context(&action)
     }
 
-    /// Apply the [`actions::SymbolsInversion`] action to this view (see [`Self::map`]).
+    /// Apply the [`actions::SymbolsInversion`] action to this view (see
+    /// [`Self::map_without_context`]).
     #[cfg(feature = "symbols")]
     pub fn invert_symbols(&mut self) -> &mut Self {
         let action = actions::SymbolsInversion::default();
 
-        self.map(&action)
+        self.map_without_context(&action)
     }
 
-    /// Apply the default [`actions::Titlecase`] action to this view (see [`Self::map`]).
+    /// Apply the default [`actions::Titlecase`] action to this view (see
+    /// [`Self::map_without_context`]).
     pub fn titlecase(&mut self) -> &mut Self {
         let action = actions::Titlecase::default();
 
-        self.map(&action)
+        self.map_without_context(&action)
     }
 
-    /// Apply the default [`actions::Upper`] action to this view (see [`Self::map`]).
+    /// Apply the default [`actions::Upper`] action to this view (see
+    /// [`Self::map_without_context`]).
     pub fn upper(&mut self) -> &mut Self {
         let action = actions::Upper::default();
 
-        self.map(&action)
+        self.map_without_context(&action)
     }
 }
 
@@ -196,7 +295,7 @@ impl<'viewee> ScopedViewBuilder<'viewee> {
     #[must_use]
     pub fn new(input: &'viewee str) -> Self {
         Self {
-            scopes: ROScopes(vec![ROScope(In(input))]),
+            scopes: ROScopes(vec![ROScope(In(input, None))]),
             viewee: input,
         }
     }
@@ -209,20 +308,14 @@ impl<'viewee> ScopedViewBuilder<'viewee> {
         self.apply_dos_line_endings_fix();
 
         ScopedView {
-            scopes: RWScopes(
-                self.scopes
-                    .0
-                    .into_iter()
-                    .map(std::convert::Into::into)
-                    .collect(),
-            ),
+            scopes: RWScopes(self.scopes.0.into_iter().map(Into::into).collect()),
         }
     }
 
     /// See [`DosFix`].
     fn apply_dos_line_endings_fix(&mut self) {
         if self.scopes.0.windows(2).any(|window| match window {
-            [ROScope(In(left)), ROScope(Out(right))] => {
+            [ROScope(In(left, ..)), ROScope(Out(right))] => {
                 left.ends_with('\r') && right.starts_with('\n')
             }
             _ => false,
@@ -254,7 +347,7 @@ impl<'viewee> ScopedViewBuilder<'viewee> {
     ///
     /// Panics if the [`Scoper`] scopes such that the view is no longer consistent, i.e.
     /// gaps were created and the original input can no longer be reconstructed from the
-    /// new view.
+    /// new view. This would be an internal bug.
     pub fn explode(&mut self, scoper: &impl Scoper) -> &mut Self {
         trace!("Exploding scopes: {:?}", self.scopes);
         let mut new = Vec::with_capacity(self.scopes.0.len());
@@ -267,7 +360,7 @@ impl<'viewee> ScopedViewBuilder<'viewee> {
             }
 
             match scope {
-                ROScope(In(s)) => {
+                ROScope(In(s, ..)) => {
                     let mut new_scopes = scoper.scope(s);
                     new_scopes.0.retain(|s| !s.is_empty());
                     new.extend(new_scopes.0);
@@ -313,9 +406,14 @@ impl<'viewee> IntoIterator for ScopedViewBuilder<'viewee> {
 
 #[cfg(test)]
 mod tests {
+    use itertools::Itertools;
+    use rstest::rstest;
+
+    use super::ScopedView;
+    use crate::scoping::scope::RWScopes;
+    use crate::scoping::scope::Scope::{self, In, Out};
     use crate::scoping::view::ScopedViewBuilder;
     use crate::RegexPattern;
-    use rstest::rstest;
 
     #[rstest]
     // Pattern only
@@ -401,10 +499,10 @@ mod tests {
     //
     #[case("Anything whatsoever gets rEkT", r".", "A")]
     #[case(
-    "Anything whatsoever gets rEkT",
-    r".*", // Greediness inverted
-    "Anything whatsoever gets rEkT"
-)]
+        "Anything whatsoever gets rEkT",
+        r".*", // Greediness inverted
+        "Anything whatsoever gets rEkT"
+    )]
     //
     // Deals with Unicode shenanigans
     #[case("😎😎", r"😎", "😎")]
@@ -425,7 +523,7 @@ mod tests {
     )]
     fn test_squeeze(#[case] input: &str, #[case] pattern: RegexPattern, #[case] expected: &str) {
         let mut builder = ScopedViewBuilder::new(input);
-        builder.explode(&crate::scoping::regex::Regex::new(pattern.clone()));
+        builder.explode(&crate::scoping::regex::Regex::new(pattern));
         let mut view = builder.build();
 
         view.squeeze();
@@ -433,4 +531,164 @@ mod tests {
 
         assert_eq!(result, expected);
     }
+
+    #[rstest]
+    #[case(
+        // New newline at all: still works
+        vec![
+            In("Hello", None),
+        ],
+        vec![
+            vec![
+                In("Hello", None),
+            ],
+        ],
+    )]
+    #[case(
+        // Single newline is fine
+        vec![
+            In("Hello\n", None),
+        ],
+        vec![
+            vec![
+                In("Hello\n", None),
+            ],
+        ],
+    )]
+    #[case(
+        // Single scope is broken up across lines properly (multiple lines in single
+        // scope)
+        vec![
+            In("Hello\nWorld", None),
+        ],
+        vec![
+            vec![
+                In("Hello\n", None),
+            ],
+            vec![
+                In("World", None),
+            ],
+        ],
+    )]
+    #[case(
+        // Single line across multiple scopes
+        vec![
+            In("Hello", None),
+            Out("World"),
+            In("!!\n", None),
+            Out(" Goodbye"),
+        ],
+        vec![
+            vec![
+                In("Hello", None),
+                Out("World"),
+                In("!!\n", None),
+            ],
+            vec![
+                Out(" Goodbye"),
+            ],
+        ],
+    )]
+    #[case(
+        // Mixed scopes & trailing newline works
+        vec![
+            In("Hello\n", None),
+            Out("World"),
+        ],
+        vec![
+            vec![
+                In("Hello\n", None),
+            ],
+            vec![
+                Out("World"),
+            ],
+        ],
+    )]
+    #[case(
+        // Mixed scopes & leading newline works
+        vec![
+            In("Hello", None),
+            Out("\nWorld"),
+        ],
+        vec![
+            vec![
+                In("Hello", None),
+                Out("\n"),
+            ],
+            vec![
+                Out("World"),
+            ],
+        ],
+    )]
+    #[case(
+        // Empty lines & empty scopes works
+        vec![
+            In("\n", None),
+            Out("\n"),
+            Out(""),
+            In("World\n", None),
+            Out(""),
+        ],
+        vec![
+            vec![
+                In("\n", None),
+            ],
+            vec![
+                Out("\n"),
+            ],
+            vec![
+                #[cfg(not(test))]Out(""), // Dropped! (do not compile)
+                In("World\n", None),
+            ],
+            #[cfg(not(test))] // Dropped! (do not compile)
+            vec![
+                Out(""),
+            ],
+        ],
+    )]
+    fn test_lines(
+        #[case] input: Vec<Scope<'_, &str>>,
+        #[case] expected: Vec<Vec<Scope<'_, &str>>>,
+    ) {
+        let view = ScopedView {
+            scopes: input.into(),
+        };
+        let result = view.lines().into_iter().collect_vec();
+        let expected = expected
+            .into_iter()
+            .map(RWScopes::from)
+            .map(ScopedView::new)
+            .collect_vec();
+
+        assert_eq!(result, expected);
+    }
+
+    #[rstest]
+    #[case(
+        "hello",
+        &["hello"] // aka: it loops at least once!
+    )]
+    #[case(
+        "hello\n",
+        &["hello\n"]
+    )]
+    #[case(
+        "hello\nworld",
+        &["hello\n", "world"]
+    )]
+    #[case(
+        "hello\nworld\n",
+        &["hello\n", "world\n"]
+    )]
+    #[case(
+        "",
+        &[] // ⚠️ no iteration happens; empty string is dropped
+    )]
+    fn test_split_inclusive(#[case] input: &str, #[case] expected: &[&str]) {
+        // This is not a useful unit test; it's just encoding and confirming the
+        // behavior of `split_inclusive`, to convince myself.
+        for (si, exp) in input.split_inclusive('\n').zip_eq(expected) {
+            assert_eq!(si, *exp);
+        }
+    }
 }
